# -*- coding: utf-8 -*-
##############################################################################
#
#    Author: Yannick Buron
#    Copyright 2013 Yannick Buron
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import openerp.addons.decimal_precision as dp

from openerp import netsvc
from openerp import pooler
from openerp import SUPERUSER_ID
from openerp.osv import fields, osv, orm
from openerp.tools.translate import _

class vote_model(osv.AbstractModel):
    ''' vote_model is meant to be inherited by any model that needs to
        use the vote systemact as a discussion topic on which messages can be attached.

        ``vote.model`` defines fields used to handle and display the
        vote history. ``vote.model`` also manages the vote_vote field which will transparently
        fill a record in vote.vote model.
    '''
    _name = 'vote.model'
    _description = 'Vote Model'

    _vote_alternative_model = False
    _vote_alternative_link_field = False
    _vote_alternative_domain = False

    def _get_vote_stats(self, cr, uid, ids, name, args, context=None):
        """ Computes:
            - vote_average: Average of all vote for thi record
            - vote_total : Number of vote for this record
            - vote_user_ids : Users which have voted for this record """

        res = {}
        vote_obj = self.pool.get('vote.vote')

        for record in self.browse(cr, uid, ids, context=context):
            res[record.id] = {}
            vote_average = []
            res[record.id]['vote_average'] = 0
            res[record.id]['vote_total'] = 0
            res[record.id]['vote_user_ids'] = []
            if self._vote_alternative_model and self._vote_alternative_link_field and self._vote_alternative_domain:
                domain = [(self._vote_alternative_link_field, '=', record.id)] + self._vote_alternative_domain
                record_ids = self.pool.get(self._vote_alternative_model).search(cr, uid, domain, context=context)
                vote_ids = vote_obj.search(cr, uid, [('model','=',self._vote_alternative_model),('res_id','in',record_ids)], context=context)
            else:
                vote_ids = vote_obj.search(cr, uid, [('model','=',self._name),('res_id','=',record.id)], context=context)

            for vote in vote_obj.browse(cr, uid, vote_ids, context=context):
                vote_average.append(vote.vote)
                res[record.id]['vote_total'] += 1
                res[record.id]['vote_user_ids'].append(vote.user_id.id)

            if len(vote_average):
                res[record.id]['vote_average'] = sum(vote_average)/len(vote_average)
        return res

    def _get_vote_vote(self, cr, uid, ids, name, arg, context=None):
        """ Display the vote of the current user for the current record, if already voted """
        res = {}
        vote_obj = self.pool.get('vote.vote')

        for record in self.browse(cr, uid, ids, context=context):
            res[record.id] = False
            vote_ids = vote_obj.search(cr, uid, [('model','=',self._name),('res_id','=',record.id),('user_id','=',uid)], context=context)
            if vote_ids:
                res[record.id] = vote_obj.browse(cr, uid, vote_ids, context=context)[0].vote
        return res

    def _set_vote_vote(self, cr, uid, id, name, value, arg, context=None):
        """ Create or update the vote in vote.vote model when we save the record """
        vote_obj = self.pool.get('vote.vote')

        vote_ids = vote_obj.search(cr, uid, [('model','=',self._name),('res_id','=',id),('user_id','=',uid)], context=context)
        if value:
            if vote_ids:
                vote_obj.write(cr, uid, vote_ids, {'vote': value}, context=context)
            else:
                vote_obj.create(cr, uid, {'model': self._name, 'res_id': id, 'user_id': uid, 'vote': value}, context=context)
        else:
            vote_obj.unlink(cr, uid, vote_ids, context=context)


    _columns = {
        'vote_average': fields.function(_get_vote_stats, type='float', string='Average vote', multi='_get_vote_stats'),
        'vote_total': fields.function(_get_vote_stats, type='integer', string='Total vote', multi='_get_vote_stats'),
        'vote_user_ids': fields.function(_get_vote_stats, type='many2many', obj='res.users', string='Vote users', multi='_get_vote_stats'),
        'vote_vote': fields.function(_get_vote_vote, fnct_inv=_set_vote_vote, type='integer', string='Vote'),
        'vote_vote_ids': fields.one2many('vote.vote', 'res_id',
            domain=lambda self: [('model', '=', self._name)],
            auto_join=True,
            string='Votes'),
    }



class vote_vote(osv.Model):
    """ vote.vote model: Contain all votes for all objects in OpenERP """

    _name = 'vote.vote'
    _description = 'Vote'

    _columns = {
        'model': fields.char('Related Document Model', size=128, select=1),
        'res_id': fields.integer('Related Document ID', select=1),
        'create_date': fields.datetime('Create date'),
        'user_id': fields.many2one('res.users', 'User', select=1),
        'vote': fields.integer('Vote')
    }

    _sql_constraints = [
        ('user_vote', 'unique(model,res_id,user_id)', 'We can only have one vote per record per user')
    ]

